//
//  ViewController.swift
//  SaveToCoreData
//
//  Created by Jon on 6/3/22.
//

import UIKit
import CoreData

class ViewController: UIViewController {

    private lazy var persistentContainer: NSPersistentContainer = {
        NSPersistentContainer(name: "User")
    }()
    
    var managedObjectContext: NSManagedObjectContext?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        persistentContainer.loadPersistentStores { [weak self] persistentStoreDescription, error in
            if let error = error {
                print("Unable to Add Persistent Store")
                print("\(error), \(error.localizedDescription)")
            } else {
                self?.getCoreUsers()
            }
            
        }
        
    }
 
    
    private func getCoreUsers(){
        let coreDataFetchRequest = NSFetchRequest<NSFetchRequestResult>(entityName: "User")
        
        let users = try? persistentContainer.viewContext.fetch(coreDataFetchRequest) as! [User]
        
        if let nonOptUsers = users {
            for user in nonOptUsers {
                print(user.username)
                print(user.name)
            }
        }
    }
    
    @IBAction func fetchUsers(_ sender: Any) {
        let url = URL(string: "https://jsonplaceholder.typicode.com/users")!
        let managedObjectContext = persistentContainer.viewContext
        URLSession.shared.fetchData(at: url){ (result: Result<[UserModel], Error>) in
            switch result {
            case .success(let users):
                for user in users {
                    let newUser = User(context: managedObjectContext)
                    newUser.website = user.website
                    newUser.name = user.name
                    newUser.username = user.username
                    newUser.phone = user.phone
                    newUser.id = Int16(user.id)
                    newUser.email = user.email
                    print(user)
                }
                do {
                    try managedObjectContext.save()
                } catch {
                    print("unable to save users to core data")
                }
            case .failure(let error):
                print(error)
            }
        }
    }
    @IBAction func deleteUsers(_ sender: Any) {
        do {
            let fetchRequest: NSFetchRequest<NSFetchRequestResult>
            fetchRequest = NSFetchRequest(entityName: "User")
            
            let deleteRequest = NSBatchDeleteRequest(
                fetchRequest: fetchRequest
                )
            
            deleteRequest.resultType = .resultTypeObjectIDs
            
            let context = persistentContainer.viewContext
            let batchDelete = try context.execute(deleteRequest) as? NSBatchDeleteResult
            
            guard let deleteResult = batchDelete?.result as? [NSManagedObjectID]
            else {return}
            
            let deletedObjects: [AnyHashable: Any] = [
                NSDeletedObjectsKey: deleteResult
            ]
            
            NSManagedObjectContext.mergeChanges(fromRemoteContextSave: deletedObjects, into: [context])
        } catch {
            
        }
    }
}

struct UserModel: Decodable {
    let id: Int
    let name: String
    let username: String
    let email: String
    let phone: String
    let website: String
}
extension URLSession {
    func fetchData(at url: URL, completion: @escaping(Result<[UserModel], Error>) -> Void) {
        self.dataTask(with: url) { (data, response, error) in
            if let error = error {
                completion(.failure(error))
            }
            
            if let data = data {
                do {
                    let users = try JSONDecoder().decode([UserModel].self, from: data)
                    completion(.success(users))
                } catch let decoderError {
                    completion(.failure(decoderError))
                }
            }
        }.resume()
    }
}
